//
//  BCIDAJSONModelError.h
//  JSONModel
//

#import <Foundation/Foundation.h>

/////////////////////////////////////////////////////////////////////////////////////////////
typedef NS_ENUM(int, kBCIDAJSONModelErrorTypes)
{
    kBCIDAJSONModelErrorInvalidData = 1,
    kBCIDAJSONModelErrorBadResponse = 2,
    kBCIDAJSONModelErrorBadJSON = 3,
    kBCIDAJSONModelErrorModelIsInvalid = 4,
    kBCIDAJSONModelErrorNilInput = 5
};

/////////////////////////////////////////////////////////////////////////////////////////////
/** The domain name used for the BCIDAJSONModelError instances */
extern NSString *const BCIDAJSONModelErrorDomain;

/**
 * If the model JSON input misses keys that are required, check the
 * userInfo dictionary of the BCIDAJSONModelError instance you get back -
 * under the bckJSONModelMissingKeys key you will find a list of the
 * names of the missing keys.
 */
extern NSString *const bckJSONModelMissingKeys;

/**
 * If JSON input has a different type than expected by the model, check the
 * userInfo dictionary of the BCIDAJSONModelError instance you get back -
 * under the bckJSONModelTypeMismatch key you will find a description
 * of the mismatched types.
 */
extern NSString *const bckJSONModelTypeMismatch;

/**
 * If an error occurs in a nested model, check the userInfo dictionary of
 * the BCIDAJSONModelError instance you get back - under the bckJSONModelKeyPath
 * key you will find key-path at which the error occurred.
 */
extern NSString *const bckJSONModelKeyPath;

/////////////////////////////////////////////////////////////////////////////////////////////
/**
 * Custom NSError subclass with shortcut methods for creating
 * the common JSONModel errors
 */
@interface BCIDAJSONModelError : NSError

@property (strong, nonatomic) NSHTTPURLResponse *httpResponse;

@property (strong, nonatomic) NSData *responseData;

/**
 * Creates a BCIDAJSONModelError instance with code kBCIDAJSONModelErrorInvalidData = 1
 */
+ (id)errorInvalidDataWithMessage:(NSString *)message;

/**
 * Creates a BCIDAJSONModelError instance with code kBCIDAJSONModelErrorInvalidData = 1
 * @param keys a set of field names that were required, but not found in the input
 */
+ (id)errorInvalidDataWithMissingKeys:(NSSet *)keys;

/**
 * Creates a BCIDAJSONModelError instance with code kBCIDAJSONModelErrorInvalidData = 1
 * @param mismatchDescription description of the type mismatch that was encountered.
 */
+ (id)errorInvalidDataWithTypeMismatch:(NSString *)mismatchDescription;

/**
 * Creates a BCIDAJSONModelError instance with code kBCIDAJSONModelErrorBadResponse = 2
 */
+ (id)errorBadResponse;

/**
 * Creates a BCIDAJSONModelError instance with code kBCIDAJSONModelErrorBadJSON = 3
 */
+ (id)errorBadJSON;

/**
 * Creates a BCIDAJSONModelError instance with code kBCIDAJSONModelErrorModelIsInvalid = 4
 */
+ (id)errorModelIsInvalid;

/**
 * Creates a BCIDAJSONModelError instance with code kBCIDAJSONModelErrorNilInput = 5
 */
+ (id)errorInputIsNil;

/**
 * Creates a new BCIDAJSONModelError with the same values plus information about the key-path of the error.
 * Properties in the new error object are the same as those from the receiver,
 * except that a new key bckJSONModelKeyPath is added to the userInfo dictionary.
 * This key contains the component string parameter. If the key is already present
 * then the new error object has the component string prepended to the existing value.
 */
- (instancetype)errorByPrependingKeyPathComponent:(NSString *)component;

/////////////////////////////////////////////////////////////////////////////////////////////
@end
